import asyncio
from argparse import ArgumentParser

import uvloop

from .shred import main
from .validators import *


# flake8: noqa: E501
def run() -> None:
    validate_environment()

    parser = ArgumentParser(description="Comprehensive DoD 5220.22-M-compliant file shredder for Linux.")
    parser.add_argument(
        "--recursive", "-r", action="store_true", help="Process directories recursively. Default is false."
    )
    parser.add_argument("--yes", "-y", action="store_true", help="Skip confirmation prompts. Default is false.")
    parser.add_argument(
        "--dryrun", "-d", action="store_true", help="Provide mock output without deleting anything. Default is false."
    )
    parser.add_argument(
        "--exhaustive",
        "-e",
        action="store_true",
        help="Exhaustively check local mounts for duplicate files by hash. Default is false.",
    )
    parser.add_argument(
        "--ignoredir",
        "-i",
        action="append",
        type=validate_file_folder,
        default=[],
        help="Specify directories to be ignored during the process. This option can be used multiple times.",
    )
    parser.add_argument(
        "--logfile", "-o", type=validate_logfile, help="Specify a file to log all output (NOT IMPLEMENTED)."
    )
    parser.add_argument("--quiet", "-q", action="store_true", help="Silence all output (NOT IMPLEMENTED).")
    parser.add_argument(
        "--verbose", "-v", action="store_true", help="Provide extra output for debugging (NOT IMPLEMENTED)."
    )
    parser.add_argument(
        "paths",
        nargs="+",
        type=validate_file_folder,
        help="Specify any number of existing files or directories to be processed.",
    )
    args = parser.parse_args()

    if sys.version_info >= (3, 11):
        with asyncio.Runner(loop_factory=uvloop.new_event_loop) as runner:
            runner.run(main(args))
    else:
        uvloop.install()
        asyncio.run(main(args))


if __name__ == "__main__":
    run()
