import os
import platform
import sys
from pathlib import Path

from aiopath import AsyncPath


def validate_file_folder(value: str) -> Path:
    file_folder_path = Path(value)
    if not file_folder_path.exists():
        raise FileNotFoundError(f"No such file or folder: {value}")
    if not file_folder_path.is_file() and not file_folder_path.is_dir():
        raise TypeError(f"Not a file or directory: {value}")
    return AsyncPath(value)


def validate_logfile(value: str) -> Path:
    logfile_path = Path(value)
    if logfile_path.exists():
        confirm = input(f"The file {value} already exists. Do you want to overwrite it? ([y]es/[n]o): ")
        if confirm.lower() not in ["yes", "y"]:
            sys.exit(f"Exiting. Log file {value} will not be overwritten.")
    return logfile_path


def validate_environment():
    if platform.system() != "Linux":
        sys.exit("Error: This script must be run on a Linux system.")

    if os.getuid() != 0:
        sys.exit("Error: This script must be run with sudo or root privileges.")
